/*
 * Public domain stuff.
 */

#include <stdio.h>
#include <math.h>
#include <unistd.h>

#define PC_ARCHI
#define PC_FPU

typedef struct {
  float r;
  float i;
} complex __attribute__ ((packed));

/* does reverse a 16bits word and does a final shift */
int bit_reverse(int x, int fs)
{
	register int r;

	r = ((x << 8) & 0xff00) | ((x >> 8) & 0x00ff);
	r = ((r << 4) & 0xf0f0) | ((r >> 4) & 0x0f0f);
	r = ((r << 2) & 0xcccc) | ((r >> 2) & 0x3333);
	r = ((r << 1) & 0xaaaa) | ((r >> 1) & 0x5555);
	r >>= fs;

	return r;
}

#define N 65536
#define LOG2N 16

short the_buf[N];
short ifft_out[N];

int reversed[N];

float get_input(int x)
{
  return (float)the_buf[x];
}

complex fft_data[N];
complex ifft_data[N];

complex fft_W[N];
complex ifft_W[N];

void ifft_butterfly(int p1, int p2, int Wind)
{
  complex mul;

  mul.r=ifft_data[p2].r * ifft_W[Wind].r - ifft_data[p2].i * ifft_W[Wind].i;
  mul.i=ifft_data[p2].r * ifft_W[Wind].i + ifft_data[p2].i * ifft_W[Wind].r;

  ifft_data[p2]=ifft_data[p1];

  ifft_data[p1].r += mul.r;
  ifft_data[p1].i += mul.i;

  ifft_data[p2].r -= mul.r;
  ifft_data[p2].i -= mul.i;
}

void ifft(void)
{
  register unsigned int i, j, n, m;

  for (i=0; i<N; i++)
    ifft_data[i]=fft_data[bit_reverse(i, 16-LOG2N)];

  for (n=N/2, m=1; n>=1; n>>=1, m<<=1)
    for (j=0; j<2*n; j+=2)
      for (i=0; i<m; i++)
	ifft_butterfly(j*m+i, (j+1)*m+i, i*n);

  for (i=0; i<N; i++) {
    register int out=ifft_data[i].r/(float)N;
    ifft_out[i]=out>32767?32767:out<-32768?-32768:out;
  }
  write(1, fft_out, 65536*2);
exit(0);

}

void init_fft(void)
{
  register unsigned int i;

  for (i=0; i<N; i++)
    fft_W[i].r=cos(-2.*M_PI*(float)i/(float)N),
    fft_W[i].i=sin(-2.*M_PI*(float)i/(float)N);

  for (i=0; i<N; i++)
    ifft_W[i].r=cos(2.*M_PI*(float)i/(float)N),
    ifft_W[i].i=sin(2.*M_PI*(float)i/(float)N);

  for (i=0; i<N; i++)
    reversed[i]=bit_reverse(i, 16-LOG2N);
}

int is_f(int x) {
  int f[][2]={{1600,400}, {3600, 300}, {5400, 300}, {-1,1}};
  int i;
  for (i=0; f[i][0]!=-1; i++)
    if (x>=f[i][0] && x<f[i][0]+f[i][1])
      return 1;
  return 0;
}

int main(void)
{
  unsigned char in[65536+44];
  register int i, j;
  FILE *f;

  init_fft();

  f=fopen("pipo.wav", "r");
  if (!f) {
    perror("fopen");
    return 1;
  }
  fread(in, 44, 1, f);
  fclose(f);

  f=fopen("compressed.data", "r");
  if (!f) {
    perror("fopen");
    return 1;
  }
  fread(in+44, 1000, 1, f);
  fclose(f);

  for (i=0, j=0; i<N; i++) {
    if (is_f(i)) {
      int c=in[j+44]<<(6+18);
      fft_data[i].i=0;
      fft_data[i].r=c>>6;
      j++;
    } else {
      fft_data[i].i=0;
      fft_data[i].r=0;
    }
  }

  ifft();

  for (i=0; i<N; i++)
    in[i+44]=(float)ifft_out[i]*255./65535.+128;

  f=fopen("tonper2.wav", "w");
  if (!f) {
    perror("fopen");
    return 1;
  }
  fwrite(in, 65536+44, 1, f);
  fclose(f);

  return 0;
}
